Struct width_counters::CounterIsize
source · pub struct CounterIsize { /* private fields */ }Expand description
An atomic counter using AtomicIsize / (core::isize).
Behavior
- The default ordering is Sequentially Consistent.
- The ordering used for atomic operations is customizable for operations ending in
with_ordering. - The choice of ordering intentionally impacts ALMOST EVERYTHING about how this counter works, including de/serialization, incrementing, decrementing, equality comparisons, partial ordering comparisons, etc.
- Total (non-partial) ordering comparisons always use the default ordering.
- Unlike the underlying AtomicIsize, this will not wrap on overflow.
Ordering
- PartialEq is implemented such that counters with differing orderings are never equal.
- PartialOrd is implemented such that counters with differing (atomic) orderings produce no (comparison) ordering.
- (Saturating) arithmetic operations are implemented such that differing atomic orderings between the operands are ignored!
Miscellaneous
You can use the to_x method to convert to any type that implements From<isize>
Implementations
sourceimpl CounterIsize
impl CounterIsize
sourcepub const MAX: isize = 2_147_483_647isize
pub const MAX: isize = 2_147_483_647isize
Largest representable value
sourcepub const MIN: isize = -2_147_483_648isize
pub const MIN: isize = -2_147_483_648isize
Smallest representable value
sourcepub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst
pub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst
Default Atomic ordering
sourcepub const fn new_with_ordering(ordering: Ordering) -> Self
pub const fn new_with_ordering(ordering: Ordering) -> Self
Instantiate with ordering
sourcepub const fn new_from_offset(offset: isize) -> Self
pub const fn new_from_offset(offset: isize) -> Self
Instantiate with offset value
sourcepub const fn new_from_offset_with_ordering(
offset: isize,
ordering: Ordering
) -> Self
pub const fn new_from_offset_with_ordering( offset: isize, ordering: Ordering ) -> Self
Instantiate with offset value and ordering
sourcepub fn get(&self) -> isize
pub fn get(&self) -> isize
Get current value with the default ordering
use width_counters::{CounterIsize as C };
use isize as U;
let c = C::new();
assert_eq!(c.get(), 0, "get returns initial value");
c.inc_one();
c.inc_one();
c.inc_one();
assert_eq!(c.get(), 3, "get returns post-increment value");sourcepub fn get_with_ordering(&self, ordering: Ordering) -> isize
pub fn get_with_ordering(&self, ordering: Ordering) -> isize
Get current value with a specific ordering
sourceimpl CounterIsize
impl CounterIsize
sourcepub fn inc_one(&self)
pub fn inc_one(&self)
Increment by one
use width_counters::{CounterIsize as C };
use core::ops::*;
use isize as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.inc_one(); });
assert_eq!(c.get(), (offset).add(20), "counter must Increment/add number of times given as per sequential ordering");
let d = C::new_from_offset(U::MAX);
d.inc_one();
d.inc_one();
assert_eq!(d.get(), U::MAX, "counter must stop at MAX ");sourcepub fn inc_one_with_ordering(&self, ordering: Ordering)
pub fn inc_one_with_ordering(&self, ordering: Ordering)
Increment by one with ordering
sourcepub fn inc_by(&self, amount: isize)
pub fn inc_by(&self, amount: isize)
Increment by specified amount
use width_counters::{CounterIsize as C };
use core::ops::*;
use isize as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.inc_by(2); });
assert_eq!((c.get() as i128).sub((20*2) as i128), ((offset) as i128), "counter must Increment by specified amount");sourcepub fn inc_by_with_ordering(&self, amount: isize, ordering: Ordering)
pub fn inc_by_with_ordering(&self, amount: isize, ordering: Ordering)
Increment by specified amount with ordering
use width_counters::{CounterIsize as C };
use isize as U;
use core::ops::*;
let m = 3isize;
let offset = U::MAX/2;
let d = C::new_from_offset(U::MAX.sub(m * 2));
d.inc_by(m);
d.inc_by(m);
d.inc_by(m);
assert_eq!(d.get(), U::MAX, "counter must stop at MAX");sourceimpl CounterIsize
impl CounterIsize
sourcepub fn dec_one(&self)
pub fn dec_one(&self)
Decrement by one
use width_counters::{CounterIsize as C };
use core::ops::*;
use isize as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.dec_one(); });
assert_eq!(c.get(), (offset).sub(20), "counter must Decrement/sub number of times given as per sequential ordering");
let d = C::new_from_offset(U::MIN);
d.dec_one();
d.dec_one();
assert_eq!(d.get(), U::MIN, "counter must stop at MIN ");sourcepub fn dec_one_with_ordering(&self, ordering: Ordering)
pub fn dec_one_with_ordering(&self, ordering: Ordering)
Decrement by one with ordering
sourcepub fn dec_by(&self, amount: isize)
pub fn dec_by(&self, amount: isize)
Decrement by specified amount
use width_counters::{CounterIsize as C };
use core::ops::*;
use isize as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.dec_by(2); });
assert_eq!((c.get() as i128).add((20*2) as i128), ((offset) as i128), "counter must Decrement by specified amount");sourcepub fn dec_by_with_ordering(&self, amount: isize, ordering: Ordering)
pub fn dec_by_with_ordering(&self, amount: isize, ordering: Ordering)
Decrement by specified amount with ordering
use width_counters::{CounterIsize as C };
use isize as U;
use core::ops::*;
let m = 3isize;
let offset = U::MAX/2;
let d = C::new_from_offset(U::MIN.add(m * 2));
d.dec_by(m);
d.dec_by(m);
d.dec_by(m);
assert_eq!(d.get(), U::MIN, "counter must stop at MIN");Trait Implementations
sourceimpl Add<CounterIsize> for CounterIsize
impl Add<CounterIsize> for CounterIsize
sourcefn add(self, rhs: Self) -> Self::Output
fn add(self, rhs: Self) -> Self::Output
- This operation is implemented with saturating arithmetic
- This operation IGNORES dissimilar atomic orderings!
type Output = CounterIsize
type Output = CounterIsize
The resulting type after applying the
+ operator.sourceimpl AsRef<Ordering> for CounterIsize
impl AsRef<Ordering> for CounterIsize
sourceimpl Clone for CounterIsize
impl Clone for CounterIsize
sourceimpl Debug for CounterIsize
impl Debug for CounterIsize
sourceimpl Default for CounterIsize
impl Default for CounterIsize
sourceimpl Display for CounterIsize
impl Display for CounterIsize
sourceimpl Div<CounterIsize> for CounterIsize
impl Div<CounterIsize> for CounterIsize
sourcefn div(self, rhs: Self) -> Self::Output
fn div(self, rhs: Self) -> Self::Output
- This operation is implemented with saturating arithmetic
- This operation IGNORES dissimilar atomic orderings!
type Output = CounterIsize
type Output = CounterIsize
The resulting type after applying the
/ operator.sourceimpl From<&CounterIsize> for isize
impl From<&CounterIsize> for isize
sourcefn from(counter: &CounterIsize) -> Self
fn from(counter: &CounterIsize) -> Self
Converts to this type from the input type.
sourceimpl From<isize> for CounterIsize
impl From<isize> for CounterIsize
sourceimpl Hash for CounterIsize
impl Hash for CounterIsize
sourceimpl Mul<CounterIsize> for CounterIsize
impl Mul<CounterIsize> for CounterIsize
sourcefn mul(self, rhs: Self) -> Self::Output
fn mul(self, rhs: Self) -> Self::Output
- This operation is implemented with saturating arithmetic
- This operation IGNORES dissimilar atomic orderings!
type Output = CounterIsize
type Output = CounterIsize
The resulting type after applying the
* operator.sourceimpl Ord for CounterIsize
impl Ord for CounterIsize
sourceimpl PartialEq<CounterIsize> for CounterIsize
impl PartialEq<CounterIsize> for CounterIsize
PartialEq is only equal when orderings are equal
use width_counters::{CounterIsize as C };
use core::sync::atomic::Ordering;
let a = C::new_from_offset_with_ordering(33, Ordering::Relaxed);
let b = C::new_from_offset_with_ordering(33, Ordering::Relaxed);
assert_eq!(a, b, "counters must be equal when counts and orderings are equal");
assert_eq!(a, b, "counters must be equal when counts and orderings are equal");
let m = 20;
(0..m).for_each(|_| { a.inc_one(); b.inc_one(); });
assert_eq!(a, b, "counters must be equal after counting same amount");
assert_eq!(a, b, "counters must be equal after counting same amount");
a.inc_one();
assert_ne!(a, b, "counters must not be equal after counting different amounts");
let c = C::new_from_offset_with_ordering(44, Ordering::Relaxed);
let d = C::new_from_offset_with_ordering(44, Ordering::Release);
assert_ne!(c, d, "ordering-inequal counters must not be equal with same count");sourceimpl PartialOrd<CounterIsize> for CounterIsize
impl PartialOrd<CounterIsize> for CounterIsize
PartialOrd only produces cmp ordering when atomic orderings are equal
use width_counters::{CounterIsize as C };
use core::sync::atomic::Ordering;
let a = C::new_from_offset_with_ordering(32, Ordering::Relaxed);
let b = C::new_from_offset_with_ordering(33, Ordering::Relaxed);
assert!(a < b, "same-cmp::ordering counters must be ordered by when counts");
assert!(b > a, "same-cmp::ordering counters must be ordered by when counts");
let m = 20;
(0..m).for_each(|_| { a.inc_one(); b.inc_one(); });
assert!(a < b, "cmp::ordering preserved after counting same amount");
assert!(b > a, "cmp::ordering preserved after counting same amount");1.0.0 · sourcefn le(&self, other: &Rhs) -> bool
fn le(&self, other: &Rhs) -> bool
This method tests less than or equal to (for
self and other) and is used by the <=
operator. Read moresourceimpl Sub<CounterIsize> for CounterIsize
impl Sub<CounterIsize> for CounterIsize
sourcefn sub(self, rhs: Self) -> Self::Output
fn sub(self, rhs: Self) -> Self::Output
- This operation is implemented with saturating arithmetic
- This operation IGNORES dissimilar atomic orderings!
type Output = CounterIsize
type Output = CounterIsize
The resulting type after applying the
- operator.impl Eq for CounterIsize