Struct width_counters::CounterU32
source · pub struct CounterU32 { /* private fields */ }Expand description
An atomic counter using AtomicU32 / (core::u32).
Behavior
- The default ordering is Sequentially Consistent.
- The ordering used for atomic operations is customizable for operations ending in
with_ordering. - The choice of ordering intentionally impacts ALMOST EVERYTHING about how this counter works, including de/serialization, incrementing, decrementing, equality comparisons, partial ordering comparisons, etc.
- Total (non-partial) ordering comparisons always use the default ordering.
- Unlike the underlying AtomicU32, this will not wrap on overflow unless the cyclic behavior mode is set.
- The default behavior is non-monotonic, so the counter can increment and decrement.
Ordering
- PartialEq is implemented such that counters with differing orderings are never equal.
- PartialOrd is implemented such that counters with differing (atomic) orderings produce no (comparison) ordering.
- (Saturating) arithmetic operations are implemented such that differing atomic orderings between the operands are ignored!
Miscellaneous
You can use the to_x method to convert to any type that implements From<u32>
Implementations
sourceimpl CounterU32
impl CounterU32
sourcepub const MAX: u32 = 4_294_967_295u32
pub const MAX: u32 = 4_294_967_295u32
Largest representable value
sourcepub const MIN: u32 = 0u32
pub const MIN: u32 = 0u32
Smallest representable value
sourcepub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst
pub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst
Default Atomic ordering
sourcepub const DEFAULT_COUNTING_BEHAVIOR: BitFlags<CountingBehavior> = _
pub const DEFAULT_COUNTING_BEHAVIOR: BitFlags<CountingBehavior> = _
Default counting behavior
sourcepub fn new_with_ordering(ordering: Ordering) -> Self
pub fn new_with_ordering(ordering: Ordering) -> Self
Instantiate with ordering
sourcepub fn new_from_offset(offset: u32) -> Self
pub fn new_from_offset(offset: u32) -> Self
Instantiate with offset value
sourcepub fn new_from_offset_with_ordering(offset: u32, ordering: Ordering) -> Self
pub fn new_from_offset_with_ordering(offset: u32, ordering: Ordering) -> Self
Instantiate with offset value and ordering
sourcepub fn set_counting_behavior<B: Into<BitFlags<CountingBehavior>>>(
&mut self,
counting_behavior: B
)
pub fn set_counting_behavior<B: Into<BitFlags<CountingBehavior>>>( &mut self, counting_behavior: B )
Set counting behavior
sourcepub fn new_with_counting_behavior<B: Into<BitFlags<CountingBehavior>>>(
counting_behavior: B
) -> Self
pub fn new_with_counting_behavior<B: Into<BitFlags<CountingBehavior>>>( counting_behavior: B ) -> Self
Instantiate with counting behaviors
sourcepub fn new_from_offset_with_counting_behavior<B: Into<BitFlags<CountingBehavior>>>(
offset: u32,
counting_behavior: B
) -> Self
pub fn new_from_offset_with_counting_behavior<B: Into<BitFlags<CountingBehavior>>>( offset: u32, counting_behavior: B ) -> Self
Instantiate with offset value and counting behavior
sourcepub fn get(&self) -> u32
pub fn get(&self) -> u32
Get current value with the default ordering
use width_counters::{ *, CounterU32 as C };
use u32 as U;
let c = C::new();
assert_eq!(c.get(), 0, "get returns initial value");
c.inc_one();
c.inc_one();
c.inc_one();
assert_eq!(c.get(), 3, "get returns post-increment value");sourcepub fn get_with_ordering(&self, ordering: Ordering) -> u32
pub fn get_with_ordering(&self, ordering: Ordering) -> u32
Get current value with a specific ordering
sourceimpl CounterU32
impl CounterU32
sourcepub fn inc_one_with_ordering(&self, ordering: Ordering)
pub fn inc_one_with_ordering(&self, ordering: Ordering)
Increment by one with ordering
sourcepub fn inc_by(&self, amount: u32)
pub fn inc_by(&self, amount: u32)
Increment by specified amount
use width_counters::{ *, CounterU32 as C };
use core::ops::*;
use u32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.inc_by(2); });
assert_eq!((c.get() as i128).sub((20*2) as i128), ((offset) as i128), "counter must Increment by specified amount");sourcepub fn inc_by_with_ordering(&self, amount: u32, ordering: Ordering)
pub fn inc_by_with_ordering(&self, amount: u32, ordering: Ordering)
Increment by specified amount with ordering
use width_counters::{ *, CounterU32 as C };
use u32 as U;
use core::ops::*;
let m = 3u32;
let d = C::new_from_offset(U::MAX.sub(m * 2));
d.inc_by(m);
d.inc_by(m);
d.inc_by(m);
assert_eq!(d.get(), U::MAX, "counter must stop at MAX");sourcepub fn get_and_inc_one(&self) -> u32
pub fn get_and_inc_one(&self) -> u32
Combine the increment (by one) and get operations
Returns the value before the increment operation
sourcepub fn get_and_inc_by(&self, amount: u32) -> u32
pub fn get_and_inc_by(&self, amount: u32) -> u32
Combine the increment (by the given amount) and get operations
Returns the value before the incrementoperation
sourcepub fn get_and_inc_by_with_ordering(
&self,
amount: u32,
ordering: Ordering
) -> u32
pub fn get_and_inc_by_with_ordering( &self, amount: u32, ordering: Ordering ) -> u32
Combine the increment (by the given amount) and get operations
Returns the value before the increment operation
sourceimpl CounterU32
impl CounterU32
sourcepub fn dec_one_with_ordering(&self, ordering: Ordering)
pub fn dec_one_with_ordering(&self, ordering: Ordering)
Decrement by one with ordering
sourcepub fn dec_by(&self, amount: u32)
pub fn dec_by(&self, amount: u32)
Decrement by specified amount
use width_counters::{ *, CounterU32 as C };
use core::ops::*;
use u32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.dec_by(2); });
assert_eq!((c.get() as i128).add((20*2) as i128), ((offset) as i128), "counter must Decrement by specified amount");sourcepub fn dec_by_with_ordering(&self, amount: u32, ordering: Ordering)
pub fn dec_by_with_ordering(&self, amount: u32, ordering: Ordering)
Decrement by specified amount with ordering
use width_counters::{ *, CounterU32 as C };
use u32 as U;
use core::ops::*;
let m = 3u32;
let d = C::new_from_offset(U::MIN.add(m * 2));
d.dec_by(m);
d.dec_by(m);
d.dec_by(m);
assert_eq!(d.get(), U::MIN, "counter must stop at MIN");sourcepub fn get_and_dec_one(&self) -> u32
pub fn get_and_dec_one(&self) -> u32
Combine the decrement (by one) and get operations
Returns the value before the decrement operation
sourcepub fn get_and_dec_by(&self, amount: u32) -> u32
pub fn get_and_dec_by(&self, amount: u32) -> u32
Combine the decrement (by the given amount) and get operations
Returns the value before the decrementoperation
sourcepub fn get_and_dec_by_with_ordering(
&self,
amount: u32,
ordering: Ordering
) -> u32
pub fn get_and_dec_by_with_ordering( &self, amount: u32, ordering: Ordering ) -> u32
Combine the decrement (by the given amount) and get operations
Returns the value before the decrement operation