pub struct CounterI32 { /* private fields */ }
An atomic counter using AtomicI32 / (core::i32).

Behavior

  1. The default ordering is Sequentially Consistent.
  2. The ordering used for atomic operations is customizable for operations ending in with_ordering.
  3. The choice of ordering intentionally impacts ALMOST EVERYTHING about how this counter works, including de/serialization, incrementing, decrementing, equality comparisons, partial ordering comparisons, etc.
  4. Total (non-partial) ordering comparisons always use the default ordering.
  5. Unlike the underlying AtomicI32, this will not wrap on overflow.

Ordering

  • PartialEq is implemented such that counters with differing orderings are never equal.
  • PartialOrd is implemented such that counters with differing (atomic) orderings produce no (comparison) ordering.
  • (Saturating) arithmetic operations are implemented such that differing atomic orderings between the operands are ignored!

Miscellaneous

You can use the to_x method to convert to any type that implements From<i32>

Implementations

source

impl CounterI32

source

pub const MAX: i32 = 2_147_483_647i32

source

pub const MIN: i32 = -2_147_483_648i32

source

pub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst

source

pub const fn new() -> Self

Instantiate

source

pub const fn new_with_ordering(ordering: Ordering) -> Self

Instantiate with ordering

source

pub const fn new_from_offset(offset: i32) -> Self

Instantiate with offset value

source

pub const fn new_from_offset_with_ordering( offset: i32, ordering: Ordering ) -> Self

Instantiate with offset value and ordering

source

pub fn get(&self) -> i32

Get current value with the default ordering

use width_counters::{CounterI32 as C };
use i32 as U;
let c = C::new();
assert_eq!(c.get(), 0, "get returns initial value");
c.inc_one();
c.inc_one();
c.inc_one();
assert_eq!(c.get(), 3, "get returns post-increment value");
source

pub fn get_with_ordering(&self, ordering: Ordering) -> i32

Get current value with a specific ordering

source

pub fn get_i128(&self) -> i128

Convenience method for getting the current value as i128

source

pub fn to_x<X: From<i32>>(&self) -> X

Convert to some type that impls From i32.

source

impl CounterI32

source

pub fn inc_one(&self)

Increment by one

use width_counters::{CounterI32 as C };
use core::ops::*; 
use i32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.inc_one(); });
assert_eq!(c.get(), (offset).add(20), "counter must Increment/add number of times given as per sequential ordering");
let d = C::new_from_offset(U::MAX);
d.inc_one();
d.inc_one();
assert_eq!(d.get(), U::MAX, "counter must stop at MAX ");
source

pub fn inc_one_with_ordering(&self, ordering: Ordering)

Increment by one with ordering

source

pub fn inc_by(&self, amount: i32)

Increment by specified amount

use width_counters::{CounterI32 as C };
use core::ops::*; 
use i32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.inc_by(2); });
assert_eq!((c.get() as i128).sub((20*2) as i128), ((offset) as i128), "counter must Increment by specified amount");
source

pub fn inc_by_with_ordering(&self, amount: i32, ordering: Ordering)

Increment by specified amount with ordering

use width_counters::{CounterI32 as C };
use i32 as U;
use core::ops::*; 
let m = 3i32;
let offset = U::MAX/2;
let d = C::new_from_offset(U::MAX.sub(m * 2));
d.inc_by(m);
d.inc_by(m);
d.inc_by(m);
assert_eq!(d.get(), U::MAX, "counter must stop at MAX");
source

impl CounterI32

source

pub fn dec_one(&self)

Decrement by one

use width_counters::{CounterI32 as C };
use core::ops::*; 
use i32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.dec_one(); });
assert_eq!(c.get(), (offset).sub(20), "counter must Decrement/sub number of times given as per sequential ordering");
let d = C::new_from_offset(U::MIN);
d.dec_one();
d.dec_one();
assert_eq!(d.get(), U::MIN, "counter must stop at MIN ");
source

pub fn dec_one_with_ordering(&self, ordering: Ordering)

Decrement by one with ordering

source

pub fn dec_by(&self, amount: i32)

Decrement by specified amount

use width_counters::{CounterI32 as C };
use core::ops::*; 
use i32 as U;
let offset = U::MAX/2;
let c = C::new_from_offset(offset);
let m = 20;
(0..m).for_each(|_| { c.dec_by(2); });
assert_eq!((c.get() as i128).add((20*2) as i128), ((offset) as i128), "counter must Decrement by specified amount");
source

pub fn dec_by_with_ordering(&self, amount: i32, ordering: Ordering)

Decrement by specified amount with ordering

use width_counters::{CounterI32 as C };
use i32 as U;
use core::ops::*; 
let m = 3i32;
let offset = U::MAX/2;
let d = C::new_from_offset(U::MIN.add(m * 2));
d.dec_by(m);
d.dec_by(m);
d.dec_by(m);
assert_eq!(d.get(), U::MIN, "counter must stop at MIN");