pub struct CounterU32 { /* private fields */ }
A monotonically increasing atomic counter using AtomicU32 / (core::u32).

Behavior

  1. The default ordering is Sequentially Consistent.
  2. The ordering used for atomic operations is customizable for operations ending in with_ordering.
  3. The choice of ordering intentionally impacts EVERYTHING about how this counter works, including de/serialization, incrementing, jumping, equality comparisons, partial ordering comparisons, etc.
  4. Total (non-partial) ordering comparisons always use the default ordering.

Other Notes

  • PartialEq is implemented such that counters with differing orderings are never equal.
  • PartialOrd is implemented such that counters with differing (atomic) orderings produce no (comparison) ordering.
  • Unlike the underlying AtomicU32, this will not wrap on overflow.

Implementations

source

impl CounterU32

source

pub const MAX: u32 = 4_294_967_295u32

source

pub const DEFAULT_ORDERING: Ordering = Ordering::SeqCst

source

pub const fn new() -> Self

Instantiate

source

pub const fn new_with_ordering(ordering: Ordering) -> Self

Instantiate with ordering

source

pub const fn new_from_offset(offset: u32) -> Self

Instantiate with offset value

source

pub const fn new_from_offset_with_ordering( offset: u32, ordering: Ordering ) -> Self

Instantiate with offset value and ordering

source

pub fn get(&self) -> u32

Get current value with the default ordering

use width_counters::{CounterU32 as C };
use u32 as U;
let c = C::new();
assert_eq!(c.get(), 0, "get returns initial value");
c.inc();
c.inc();
c.inc();
assert_eq!(c.get(), 3, "get returns post-increment value");
source

pub fn get_with_ordering(&self, ordering: Ordering) -> u32

Get current value with a specific ordering

source

pub fn inc(&self)

Increment by one

use width_counters::{CounterU32 as C };
use u32 as U;
let c = C::new();
let m = 20;
(0..m).for_each(|_| { c.inc(); });
assert_eq!(c.get(), 20, "counter must count number of times given as per sequential ordering");
let d = C::new_from_offset(U::MAX);
d.inc();
d.inc();
assert_eq!(d.get(), U::MAX, "counter must stop at maximum");
source

pub fn inc_with_ordering(&self, ordering: Ordering)

Increment by one with ordering

source

pub fn jump(&self, amount: u32)

Jump by specified amount

use width_counters::{CounterU32 as C };
use u32 as U;
let c = C::default();
let m = 20;
(0..m).for_each(|_| { c.jump(2); });
assert_eq!(c.get(), 20*2, "counter must jump by specified amount");
source

pub fn jump_with_ordering(&self, amount: u32, ordering: Ordering)

Jump by specified amount with ordering

use width_counters::{CounterU32 as C };
use u32 as U;
let c = C::new();
let m: U = 20;
(0..m).for_each(|_| { c.jump(2); });
assert_eq!(c.get(), 40, "counter must jump by specified amount");
let d = C::new_from_offset(U::MAX - m);
d.jump(m/3);
d.jump(m/3);
d.jump(m/2);
assert_eq!(d.get(), U::MAX, "counter must stop at maximum");